'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _inflection = require('inflection');

var _queryBuilder = require('./query-builder');

var _queryBuilder2 = _interopRequireDefault(_queryBuilder);

var _relation = require('./relation');

var _relation2 = _interopRequireDefault(_relation);

var _relationType = require('./enums/relation-type');

var _relationType2 = _interopRequireDefault(_relationType);

var _errors = require('./errors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Base Model class which shall be extended by the attributes of a database
 * object.
 */

var Model = function () {
  _createClass(Model, null, [{
    key: 'tableName',

    /**
     * Case-sensitive name of the database table which corresponds to the Model.
     * @type {string}
     */
    get: function get() {
      return (0, _inflection.tableize)(this.name);
    }

    /**
     * ID attribute, which is used as the primary key of the Model.
     * @type {string}
     */

  }, {
    key: 'idAttribute',
    get: function get() {
      return 'id';
    }

    /**
     * Creates a new Model instance.
     * @param {Object} [props={}] Initial properties of the instance.
     * @param {boolean} [isNew=true] Determines whether the "props" of the
     * instance are considered new.
     */

  }]);

  function Model() {
    var props = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    var isNew = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

    _classCallCheck(this, Model);

    // Set the initial properties of the instance
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = Object.keys(props)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var key = _step.value;

        this[key] = props[key];
      }

      // Initialize a store for old properties of the instance
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    Object.defineProperty(this, '_oldProps', {
      writable: true,
      value: isNew ? {} : Object.assign({}, props)
    });
  }

  /**
   * Returns a new QueryBuilder instance which corresponds to the current Model.
   * @returns {QueryBuilder}
   */


  _createClass(Model, [{
    key: 'fetchRelated',


    /**
     * Fetches the given related Models of the current instance.
     * @param {...string} props Relation attributes to be fetched.
     * @returns {QueryBuilder}
     */
    value: function fetchRelated() {
      var qb = this._getQueryBuilder();
      if (!qb) throw new _errors.InexistentDbObjectError();

      return qb.withRelated.apply(qb, arguments);
    }

    /**
     * Queues the deletion of the current Model from the database.
     * @throws {InexistentDbObjectError}
     * @returns {QueryBuilder}
     */

  }, {
    key: 'del',
    value: function del() {
      var qb = this._getQueryBuilder();
      if (!qb) throw new _errors.InexistentDbObjectError();

      return qb.del();
    }

    /**
     * Queues saving (creating or updating) the current Model in the database.
     * If the 'idAttribute' of the current instance is set, then this method
     * queues an update query based on it. Otherwise, a new Model gets inserted
     * into the database.
     * @throws {EmptyDbObjectError}
     * @returns {QueryBuilder}
     */

  }, {
    key: 'save',
    value: function save() {
      var qb = this._getQueryBuilder();
      var ignorableProps = [this.constructor.idAttribute];
      var updatableProps = {};

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = Object.keys(this)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var key = _step2.value;

          // Respect ignorable properties
          if (ignorableProps.includes(key)) continue;

          var oldValue = this._oldProps[key];
          var newValue = this[key];

          // New and modified properties must be updated
          if (typeof oldValue === 'undefined' || newValue !== oldValue) {
            updatableProps[key] = newValue;
          }
        }

        // Don't run unnecessary queries
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      if (Object.keys(updatableProps).length === 0) {
        if (!qb) {
          throw new _errors.EmptyDbObjectError();
        }

        return qb;
      }

      // Update the Model's old properties with the new ones
      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = Object.keys(updatableProps)[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var _key = _step3.value;

          this._oldProps[_key] = updatableProps[_key];
        }

        // Check whether the current instance needs to be given an ID
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      if (!qb) {
        return this.constructor.query().insert(updatableProps);
      }

      return qb.update(updatableProps);
    }

    /**
     * @returns {?QueryBuilder}
     * @private
     */

  }, {
    key: '_getQueryBuilder',
    value: function _getQueryBuilder() {
      var idAttribute = this.constructor.idAttribute;
      if (typeof this[idAttribute] === 'undefined') return null;

      return this.constructor.query().where(_defineProperty({}, idAttribute, this[idAttribute]));
    }
  }], [{
    key: 'query',
    value: function query() {
      return new _queryBuilder2.default(this);
    }

    /**
     * Creates a one-to-one relation between the current Model and a target.
     * @param {string|Model} Target Name or static reference to the joinable
     * table's Model.
     * @param {string} [foreignKey] Foreign key in the target Model.
     * @returns {Relation}
     */

  }, {
    key: 'hasOne',
    value: function hasOne(Target, foreignKey) {
      return new _relation2.default(this, Target, _relationType2.default.ONE_TO_ONE, foreignKey);
    }

    /**
     * Creates a one-to-many relation between the current Model and a target.
     * @param {string|Model} Target Name or static reference to the joinable
     * table's Model.
     * @param {string} [foreignKey] Foreign key in the target Model.
     * @returns {Relation}
     */

  }, {
    key: 'hasMany',
    value: function hasMany(Target, foreignKey) {
      return new _relation2.default(this, Target, _relationType2.default.ONE_TO_MANY, foreignKey);
    }

    /**
     * Creates a many-to-one relation between the current Model and a target.
     * @param {string|Model} Target Name or static reference to the joinable
     * table's Model.
     * @param {string} [foreignKey] Foreign key in this Model.
     * @returns {Relation}
     */

  }, {
    key: 'belongsTo',
    value: function belongsTo(Target, foreignKey) {
      return new _relation2.default(this, Target, _relationType2.default.MANY_TO_ONE, foreignKey);
    }
  }]);

  return Model;
}();

exports.default = Model;
//# sourceMappingURL=model.js.map